// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: exampleProtoAcme.proto

package com.acme.glup;

public final class ExampleProtoAcme {
  private ExampleProtoAcme() {}
  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistryLite registry) {
  }

  public static void registerAllExtensions(
      com.google.protobuf.ExtensionRegistry registry) {
    registerAllExtensions(
        (com.google.protobuf.ExtensionRegistryLite) registry);
  }
  public interface ClickCasOrBuilder extends
      // @@protoc_insertion_point(interface_extends:Acme.Glup.ClickCas)
      com.google.protobuf.MessageOrBuilder {

    /**
     * <code>.Acme.Glup.Origin glup_origin = 1;</code>
     * @return Whether the glupOrigin field is set.
     */
    boolean hasGlupOrigin();
    /**
     * <code>.Acme.Glup.Origin glup_origin = 1;</code>
     * @return The glupOrigin.
     */
    com.acme.glup.MetadataProto.Origin getGlupOrigin();
    /**
     * <code>.Acme.Glup.Origin glup_origin = 1;</code>
     */
    com.acme.glup.MetadataProto.OriginOrBuilder getGlupOriginOrBuilder();

    /**
     * <code>.Acme.Glup.Partition partition = 2;</code>
     * @return Whether the partition field is set.
     */
    boolean hasPartition();
    /**
     * <code>.Acme.Glup.Partition partition = 2;</code>
     * @return The partition.
     */
    com.acme.glup.MetadataProto.Partition getPartition();
    /**
     * <code>.Acme.Glup.Partition partition = 2;</code>
     */
    com.acme.glup.MetadataProto.PartitionOrBuilder getPartitionOrBuilder();

    /**
     * <code>string uid = 5;</code>
     * @return The uid.
     */
    java.lang.String getUid();
    /**
     * <code>string uid = 5;</code>
     * @return The bytes for uid.
     */
    com.google.protobuf.ByteString
        getUidBytes();

    /**
     * <pre>
     * Schema field
     * (...)
     * </pre>
     *
     * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
     */
    int getSetFieldsCount();
    /**
     * <pre>
     * Schema field
     * (...)
     * </pre>
     *
     * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
     */
    boolean containsSetFields(
        int key);
    /**
     * Use {@link #getSetFieldsMap()} instead.
     */
    @java.lang.Deprecated
    java.util.Map<java.lang.Integer, java.lang.Boolean>
    getSetFields();
    /**
     * <pre>
     * Schema field
     * (...)
     * </pre>
     *
     * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
     */
    java.util.Map<java.lang.Integer, java.lang.Boolean>
    getSetFieldsMap();
    /**
     * <pre>
     * Schema field
     * (...)
     * </pre>
     *
     * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
     */

    boolean getSetFieldsOrDefault(
        int key,
        boolean defaultValue);
    /**
     * <pre>
     * Schema field
     * (...)
     * </pre>
     *
     * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
     */

    boolean getSetFieldsOrThrow(
        int key);

    /**
     * <pre>
     * standard glup field
     * </pre>
     *
     * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
     */
    java.util.List<com.acme.glup.MetadataProto.ControlMessage.Watermark> 
        getControlMessageList();
    /**
     * <pre>
     * standard glup field
     * </pre>
     *
     * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
     */
    com.acme.glup.MetadataProto.ControlMessage.Watermark getControlMessage(int index);
    /**
     * <pre>
     * standard glup field
     * </pre>
     *
     * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
     */
    int getControlMessageCount();
    /**
     * <pre>
     * standard glup field
     * </pre>
     *
     * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
     */
    java.util.List<? extends com.acme.glup.MetadataProto.ControlMessage.WatermarkOrBuilder> 
        getControlMessageOrBuilderList();
    /**
     * <pre>
     * standard glup field
     * </pre>
     *
     * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
     */
    com.acme.glup.MetadataProto.ControlMessage.WatermarkOrBuilder getControlMessageOrBuilder(
        int index);
  }
  /**
   * Protobuf type {@code Acme.Glup.ClickCas}
   */
  public  static final class ClickCas extends
      com.google.protobuf.GeneratedMessageV3 implements
      // @@protoc_insertion_point(message_implements:Acme.Glup.ClickCas)
      ClickCasOrBuilder {
  private static final long serialVersionUID = 0L;
    // Use ClickCas.newBuilder() to construct.
    private ClickCas(com.google.protobuf.GeneratedMessageV3.Builder<?> builder) {
      super(builder);
    }
    private ClickCas() {
      uid_ = "";
      controlMessage_ = java.util.Collections.emptyList();
    }

    @java.lang.Override
    @SuppressWarnings({"unused"})
    protected java.lang.Object newInstance(
        UnusedPrivateParameter unused) {
      return new ClickCas();
    }

    @java.lang.Override
    public final com.google.protobuf.UnknownFieldSet
    getUnknownFields() {
      return this.unknownFields;
    }
    private ClickCas(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      this();
      if (extensionRegistry == null) {
        throw new java.lang.NullPointerException();
      }
      int mutable_bitField0_ = 0;
      com.google.protobuf.UnknownFieldSet.Builder unknownFields =
          com.google.protobuf.UnknownFieldSet.newBuilder();
      try {
        boolean done = false;
        while (!done) {
          int tag = input.readTag();
          switch (tag) {
            case 0:
              done = true;
              break;
            case 10: {
              com.acme.glup.MetadataProto.Origin.Builder subBuilder = null;
              if (glupOrigin_ != null) {
                subBuilder = glupOrigin_.toBuilder();
              }
              glupOrigin_ = input.readMessage(com.acme.glup.MetadataProto.Origin.parser(), extensionRegistry);
              if (subBuilder != null) {
                subBuilder.mergeFrom(glupOrigin_);
                glupOrigin_ = subBuilder.buildPartial();
              }

              break;
            }
            case 18: {
              com.acme.glup.MetadataProto.Partition.Builder subBuilder = null;
              if (partition_ != null) {
                subBuilder = partition_.toBuilder();
              }
              partition_ = input.readMessage(com.acme.glup.MetadataProto.Partition.parser(), extensionRegistry);
              if (subBuilder != null) {
                subBuilder.mergeFrom(partition_);
                partition_ = subBuilder.buildPartial();
              }

              break;
            }
            case 42: {
              java.lang.String s = input.readStringRequireUtf8();

              uid_ = s;
              break;
            }
            case 400082: {
              if (!((mutable_bitField0_ & 0x00000001) != 0)) {
                setFields_ = com.google.protobuf.MapField.newMapField(
                    SetFieldsDefaultEntryHolder.defaultEntry);
                mutable_bitField0_ |= 0x00000001;
              }
              com.google.protobuf.MapEntry<java.lang.Integer, java.lang.Boolean>
              setFields__ = input.readMessage(
                  SetFieldsDefaultEntryHolder.defaultEntry.getParserForType(), extensionRegistry);
              setFields_.getMutableMap().put(
                  setFields__.getKey(), setFields__.getValue());
              break;
            }
            case 16777210: {
              if (!((mutable_bitField0_ & 0x00000002) != 0)) {
                controlMessage_ = new java.util.ArrayList<com.acme.glup.MetadataProto.ControlMessage.Watermark>();
                mutable_bitField0_ |= 0x00000002;
              }
              controlMessage_.add(
                  input.readMessage(com.acme.glup.MetadataProto.ControlMessage.Watermark.parser(), extensionRegistry));
              break;
            }
            default: {
              if (!parseUnknownField(
                  input, unknownFields, extensionRegistry, tag)) {
                done = true;
              }
              break;
            }
          }
        }
      } catch (com.google.protobuf.InvalidProtocolBufferException e) {
        throw e.setUnfinishedMessage(this);
      } catch (java.io.IOException e) {
        throw new com.google.protobuf.InvalidProtocolBufferException(
            e).setUnfinishedMessage(this);
      } finally {
        if (((mutable_bitField0_ & 0x00000002) != 0)) {
          controlMessage_ = java.util.Collections.unmodifiableList(controlMessage_);
        }
        this.unknownFields = unknownFields.build();
        makeExtensionsImmutable();
      }
    }
    public static final com.google.protobuf.Descriptors.Descriptor
        getDescriptor() {
      return com.acme.glup.ExampleProtoAcme.internal_static_Acme_Glup_ClickCas_descriptor;
    }

    @SuppressWarnings({"rawtypes"})
    @java.lang.Override
    protected com.google.protobuf.MapField internalGetMapField(
        int number) {
      switch (number) {
        case 50010:
          return internalGetSetFields();
        default:
          throw new RuntimeException(
              "Invalid map field number: " + number);
      }
    }
    @java.lang.Override
    protected com.google.protobuf.GeneratedMessageV3.FieldAccessorTable
        internalGetFieldAccessorTable() {
      return com.acme.glup.ExampleProtoAcme.internal_static_Acme_Glup_ClickCas_fieldAccessorTable
          .ensureFieldAccessorsInitialized(
              com.acme.glup.ExampleProtoAcme.ClickCas.class, com.acme.glup.ExampleProtoAcme.ClickCas.Builder.class);
    }

    public static final int GLUP_ORIGIN_FIELD_NUMBER = 1;
    private com.acme.glup.MetadataProto.Origin glupOrigin_;
    /**
     * <code>.Acme.Glup.Origin glup_origin = 1;</code>
     * @return Whether the glupOrigin field is set.
     */
    public boolean hasGlupOrigin() {
      return glupOrigin_ != null;
    }
    /**
     * <code>.Acme.Glup.Origin glup_origin = 1;</code>
     * @return The glupOrigin.
     */
    public com.acme.glup.MetadataProto.Origin getGlupOrigin() {
      return glupOrigin_ == null ? com.acme.glup.MetadataProto.Origin.getDefaultInstance() : glupOrigin_;
    }
    /**
     * <code>.Acme.Glup.Origin glup_origin = 1;</code>
     */
    public com.acme.glup.MetadataProto.OriginOrBuilder getGlupOriginOrBuilder() {
      return getGlupOrigin();
    }

    public static final int PARTITION_FIELD_NUMBER = 2;
    private com.acme.glup.MetadataProto.Partition partition_;
    /**
     * <code>.Acme.Glup.Partition partition = 2;</code>
     * @return Whether the partition field is set.
     */
    public boolean hasPartition() {
      return partition_ != null;
    }
    /**
     * <code>.Acme.Glup.Partition partition = 2;</code>
     * @return The partition.
     */
    public com.acme.glup.MetadataProto.Partition getPartition() {
      return partition_ == null ? com.acme.glup.MetadataProto.Partition.getDefaultInstance() : partition_;
    }
    /**
     * <code>.Acme.Glup.Partition partition = 2;</code>
     */
    public com.acme.glup.MetadataProto.PartitionOrBuilder getPartitionOrBuilder() {
      return getPartition();
    }

    public static final int UID_FIELD_NUMBER = 5;
    private volatile java.lang.Object uid_;
    /**
     * <code>string uid = 5;</code>
     * @return The uid.
     */
    public java.lang.String getUid() {
      java.lang.Object ref = uid_;
      if (ref instanceof java.lang.String) {
        return (java.lang.String) ref;
      } else {
        com.google.protobuf.ByteString bs = 
            (com.google.protobuf.ByteString) ref;
        java.lang.String s = bs.toStringUtf8();
        uid_ = s;
        return s;
      }
    }
    /**
     * <code>string uid = 5;</code>
     * @return The bytes for uid.
     */
    public com.google.protobuf.ByteString
        getUidBytes() {
      java.lang.Object ref = uid_;
      if (ref instanceof java.lang.String) {
        com.google.protobuf.ByteString b = 
            com.google.protobuf.ByteString.copyFromUtf8(
                (java.lang.String) ref);
        uid_ = b;
        return b;
      } else {
        return (com.google.protobuf.ByteString) ref;
      }
    }

    public static final int SET_FIELDS_FIELD_NUMBER = 50010;
    private static final class SetFieldsDefaultEntryHolder {
      static final com.google.protobuf.MapEntry<
          java.lang.Integer, java.lang.Boolean> defaultEntry =
              com.google.protobuf.MapEntry
              .<java.lang.Integer, java.lang.Boolean>newDefaultInstance(
                  com.acme.glup.ExampleProtoAcme.internal_static_Acme_Glup_ClickCas_SetFieldsEntry_descriptor, 
                  com.google.protobuf.WireFormat.FieldType.INT32,
                  0,
                  com.google.protobuf.WireFormat.FieldType.BOOL,
                  false);
    }
    private com.google.protobuf.MapField<
        java.lang.Integer, java.lang.Boolean> setFields_;
    private com.google.protobuf.MapField<java.lang.Integer, java.lang.Boolean>
    internalGetSetFields() {
      if (setFields_ == null) {
        return com.google.protobuf.MapField.emptyMapField(
            SetFieldsDefaultEntryHolder.defaultEntry);
      }
      return setFields_;
    }

    public int getSetFieldsCount() {
      return internalGetSetFields().getMap().size();
    }
    /**
     * <pre>
     * Schema field
     * (...)
     * </pre>
     *
     * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
     */

    public boolean containsSetFields(
        int key) {
      
      return internalGetSetFields().getMap().containsKey(key);
    }
    /**
     * Use {@link #getSetFieldsMap()} instead.
     */
    @java.lang.Deprecated
    public java.util.Map<java.lang.Integer, java.lang.Boolean> getSetFields() {
      return getSetFieldsMap();
    }
    /**
     * <pre>
     * Schema field
     * (...)
     * </pre>
     *
     * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
     */

    public java.util.Map<java.lang.Integer, java.lang.Boolean> getSetFieldsMap() {
      return internalGetSetFields().getMap();
    }
    /**
     * <pre>
     * Schema field
     * (...)
     * </pre>
     *
     * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
     */

    public boolean getSetFieldsOrDefault(
        int key,
        boolean defaultValue) {
      
      java.util.Map<java.lang.Integer, java.lang.Boolean> map =
          internalGetSetFields().getMap();
      return map.containsKey(key) ? map.get(key) : defaultValue;
    }
    /**
     * <pre>
     * Schema field
     * (...)
     * </pre>
     *
     * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
     */

    public boolean getSetFieldsOrThrow(
        int key) {
      
      java.util.Map<java.lang.Integer, java.lang.Boolean> map =
          internalGetSetFields().getMap();
      if (!map.containsKey(key)) {
        throw new java.lang.IllegalArgumentException();
      }
      return map.get(key);
    }

    public static final int CONTROL_MESSAGE_FIELD_NUMBER = 2097151;
    private java.util.List<com.acme.glup.MetadataProto.ControlMessage.Watermark> controlMessage_;
    /**
     * <pre>
     * standard glup field
     * </pre>
     *
     * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
     */
    public java.util.List<com.acme.glup.MetadataProto.ControlMessage.Watermark> getControlMessageList() {
      return controlMessage_;
    }
    /**
     * <pre>
     * standard glup field
     * </pre>
     *
     * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
     */
    public java.util.List<? extends com.acme.glup.MetadataProto.ControlMessage.WatermarkOrBuilder> 
        getControlMessageOrBuilderList() {
      return controlMessage_;
    }
    /**
     * <pre>
     * standard glup field
     * </pre>
     *
     * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
     */
    public int getControlMessageCount() {
      return controlMessage_.size();
    }
    /**
     * <pre>
     * standard glup field
     * </pre>
     *
     * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
     */
    public com.acme.glup.MetadataProto.ControlMessage.Watermark getControlMessage(int index) {
      return controlMessage_.get(index);
    }
    /**
     * <pre>
     * standard glup field
     * </pre>
     *
     * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
     */
    public com.acme.glup.MetadataProto.ControlMessage.WatermarkOrBuilder getControlMessageOrBuilder(
        int index) {
      return controlMessage_.get(index);
    }

    private byte memoizedIsInitialized = -1;
    @java.lang.Override
    public final boolean isInitialized() {
      byte isInitialized = memoizedIsInitialized;
      if (isInitialized == 1) return true;
      if (isInitialized == 0) return false;

      memoizedIsInitialized = 1;
      return true;
    }

    @java.lang.Override
    public void writeTo(com.google.protobuf.CodedOutputStream output)
                        throws java.io.IOException {
      if (glupOrigin_ != null) {
        output.writeMessage(1, getGlupOrigin());
      }
      if (partition_ != null) {
        output.writeMessage(2, getPartition());
      }
      if (!getUidBytes().isEmpty()) {
        com.google.protobuf.GeneratedMessageV3.writeString(output, 5, uid_);
      }
      com.google.protobuf.GeneratedMessageV3
        .serializeIntegerMapTo(
          output,
          internalGetSetFields(),
          SetFieldsDefaultEntryHolder.defaultEntry,
          50010);
      for (int i = 0; i < controlMessage_.size(); i++) {
        output.writeMessage(2097151, controlMessage_.get(i));
      }
      unknownFields.writeTo(output);
    }

    @java.lang.Override
    public int getSerializedSize() {
      int size = memoizedSize;
      if (size != -1) return size;

      size = 0;
      if (glupOrigin_ != null) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(1, getGlupOrigin());
      }
      if (partition_ != null) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(2, getPartition());
      }
      if (!getUidBytes().isEmpty()) {
        size += com.google.protobuf.GeneratedMessageV3.computeStringSize(5, uid_);
      }
      for (java.util.Map.Entry<java.lang.Integer, java.lang.Boolean> entry
           : internalGetSetFields().getMap().entrySet()) {
        com.google.protobuf.MapEntry<java.lang.Integer, java.lang.Boolean>
        setFields__ = SetFieldsDefaultEntryHolder.defaultEntry.newBuilderForType()
            .setKey(entry.getKey())
            .setValue(entry.getValue())
            .build();
        size += com.google.protobuf.CodedOutputStream
            .computeMessageSize(50010, setFields__);
      }
      for (int i = 0; i < controlMessage_.size(); i++) {
        size += com.google.protobuf.CodedOutputStream
          .computeMessageSize(2097151, controlMessage_.get(i));
      }
      size += unknownFields.getSerializedSize();
      memoizedSize = size;
      return size;
    }

    @java.lang.Override
    public boolean equals(final java.lang.Object obj) {
      if (obj == this) {
       return true;
      }
      if (!(obj instanceof com.acme.glup.ExampleProtoAcme.ClickCas)) {
        return super.equals(obj);
      }
      com.acme.glup.ExampleProtoAcme.ClickCas other = (com.acme.glup.ExampleProtoAcme.ClickCas) obj;

      if (hasGlupOrigin() != other.hasGlupOrigin()) return false;
      if (hasGlupOrigin()) {
        if (!getGlupOrigin()
            .equals(other.getGlupOrigin())) return false;
      }
      if (hasPartition() != other.hasPartition()) return false;
      if (hasPartition()) {
        if (!getPartition()
            .equals(other.getPartition())) return false;
      }
      if (!getUid()
          .equals(other.getUid())) return false;
      if (!internalGetSetFields().equals(
          other.internalGetSetFields())) return false;
      if (!getControlMessageList()
          .equals(other.getControlMessageList())) return false;
      if (!unknownFields.equals(other.unknownFields)) return false;
      return true;
    }

    @java.lang.Override
    public int hashCode() {
      if (memoizedHashCode != 0) {
        return memoizedHashCode;
      }
      int hash = 41;
      hash = (19 * hash) + getDescriptor().hashCode();
      if (hasGlupOrigin()) {
        hash = (37 * hash) + GLUP_ORIGIN_FIELD_NUMBER;
        hash = (53 * hash) + getGlupOrigin().hashCode();
      }
      if (hasPartition()) {
        hash = (37 * hash) + PARTITION_FIELD_NUMBER;
        hash = (53 * hash) + getPartition().hashCode();
      }
      hash = (37 * hash) + UID_FIELD_NUMBER;
      hash = (53 * hash) + getUid().hashCode();
      if (!internalGetSetFields().getMap().isEmpty()) {
        hash = (37 * hash) + SET_FIELDS_FIELD_NUMBER;
        hash = (53 * hash) + internalGetSetFields().hashCode();
      }
      if (getControlMessageCount() > 0) {
        hash = (37 * hash) + CONTROL_MESSAGE_FIELD_NUMBER;
        hash = (53 * hash) + getControlMessageList().hashCode();
      }
      hash = (29 * hash) + unknownFields.hashCode();
      memoizedHashCode = hash;
      return hash;
    }

    public static com.acme.glup.ExampleProtoAcme.ClickCas parseFrom(
        java.nio.ByteBuffer data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseFrom(
        java.nio.ByteBuffer data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseFrom(
        com.google.protobuf.ByteString data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseFrom(
        com.google.protobuf.ByteString data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseFrom(byte[] data)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseFrom(
        byte[] data,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws com.google.protobuf.InvalidProtocolBufferException {
      return PARSER.parseFrom(data, extensionRegistry);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseDelimitedFrom(java.io.InputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseDelimitedWithIOException(PARSER, input);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseDelimitedFrom(
        java.io.InputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseDelimitedWithIOException(PARSER, input, extensionRegistry);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseFrom(
        com.google.protobuf.CodedInputStream input)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input);
    }
    public static com.acme.glup.ExampleProtoAcme.ClickCas parseFrom(
        com.google.protobuf.CodedInputStream input,
        com.google.protobuf.ExtensionRegistryLite extensionRegistry)
        throws java.io.IOException {
      return com.google.protobuf.GeneratedMessageV3
          .parseWithIOException(PARSER, input, extensionRegistry);
    }

    @java.lang.Override
    public Builder newBuilderForType() { return newBuilder(); }
    public static Builder newBuilder() {
      return DEFAULT_INSTANCE.toBuilder();
    }
    public static Builder newBuilder(com.acme.glup.ExampleProtoAcme.ClickCas prototype) {
      return DEFAULT_INSTANCE.toBuilder().mergeFrom(prototype);
    }
    @java.lang.Override
    public Builder toBuilder() {
      return this == DEFAULT_INSTANCE
          ? new Builder() : new Builder().mergeFrom(this);
    }

    @java.lang.Override
    protected Builder newBuilderForType(
        com.google.protobuf.GeneratedMessageV3.BuilderParent parent) {
      Builder builder = new Builder(parent);
      return builder;
    }
    /**
     * Protobuf type {@code Acme.Glup.ClickCas}
     */
    public static final class Builder extends
        com.google.protobuf.GeneratedMessageV3.Builder<Builder> implements
        // @@protoc_insertion_point(builder_implements:Acme.Glup.ClickCas)
        com.acme.glup.ExampleProtoAcme.ClickCasOrBuilder {
      public static final com.google.protobuf.Descriptors.Descriptor
          getDescriptor() {
        return com.acme.glup.ExampleProtoAcme.internal_static_Acme_Glup_ClickCas_descriptor;
      }

      @SuppressWarnings({"rawtypes"})
      protected com.google.protobuf.MapField internalGetMapField(
          int number) {
        switch (number) {
          case 50010:
            return internalGetSetFields();
          default:
            throw new RuntimeException(
                "Invalid map field number: " + number);
        }
      }
      @SuppressWarnings({"rawtypes"})
      protected com.google.protobuf.MapField internalGetMutableMapField(
          int number) {
        switch (number) {
          case 50010:
            return internalGetMutableSetFields();
          default:
            throw new RuntimeException(
                "Invalid map field number: " + number);
        }
      }
      @java.lang.Override
      protected com.google.protobuf.GeneratedMessageV3.FieldAccessorTable
          internalGetFieldAccessorTable() {
        return com.acme.glup.ExampleProtoAcme.internal_static_Acme_Glup_ClickCas_fieldAccessorTable
            .ensureFieldAccessorsInitialized(
                com.acme.glup.ExampleProtoAcme.ClickCas.class, com.acme.glup.ExampleProtoAcme.ClickCas.Builder.class);
      }

      // Construct using com.acme.glup.ExampleProtoAcme.ClickCas.newBuilder()
      private Builder() {
        maybeForceBuilderInitialization();
      }

      private Builder(
          com.google.protobuf.GeneratedMessageV3.BuilderParent parent) {
        super(parent);
        maybeForceBuilderInitialization();
      }
      private void maybeForceBuilderInitialization() {
        if (com.google.protobuf.GeneratedMessageV3
                .alwaysUseFieldBuilders) {
          getControlMessageFieldBuilder();
        }
      }
      @java.lang.Override
      public Builder clear() {
        super.clear();
        if (glupOriginBuilder_ == null) {
          glupOrigin_ = null;
        } else {
          glupOrigin_ = null;
          glupOriginBuilder_ = null;
        }
        if (partitionBuilder_ == null) {
          partition_ = null;
        } else {
          partition_ = null;
          partitionBuilder_ = null;
        }
        uid_ = "";

        internalGetMutableSetFields().clear();
        if (controlMessageBuilder_ == null) {
          controlMessage_ = java.util.Collections.emptyList();
          bitField0_ = (bitField0_ & ~0x00000002);
        } else {
          controlMessageBuilder_.clear();
        }
        return this;
      }

      @java.lang.Override
      public com.google.protobuf.Descriptors.Descriptor
          getDescriptorForType() {
        return com.acme.glup.ExampleProtoAcme.internal_static_Acme_Glup_ClickCas_descriptor;
      }

      @java.lang.Override
      public com.acme.glup.ExampleProtoAcme.ClickCas getDefaultInstanceForType() {
        return com.acme.glup.ExampleProtoAcme.ClickCas.getDefaultInstance();
      }

      @java.lang.Override
      public com.acme.glup.ExampleProtoAcme.ClickCas build() {
        com.acme.glup.ExampleProtoAcme.ClickCas result = buildPartial();
        if (!result.isInitialized()) {
          throw newUninitializedMessageException(result);
        }
        return result;
      }

      @java.lang.Override
      public com.acme.glup.ExampleProtoAcme.ClickCas buildPartial() {
        com.acme.glup.ExampleProtoAcme.ClickCas result = new com.acme.glup.ExampleProtoAcme.ClickCas(this);
        int from_bitField0_ = bitField0_;
        if (glupOriginBuilder_ == null) {
          result.glupOrigin_ = glupOrigin_;
        } else {
          result.glupOrigin_ = glupOriginBuilder_.build();
        }
        if (partitionBuilder_ == null) {
          result.partition_ = partition_;
        } else {
          result.partition_ = partitionBuilder_.build();
        }
        result.uid_ = uid_;
        result.setFields_ = internalGetSetFields();
        result.setFields_.makeImmutable();
        if (controlMessageBuilder_ == null) {
          if (((bitField0_ & 0x00000002) != 0)) {
            controlMessage_ = java.util.Collections.unmodifiableList(controlMessage_);
            bitField0_ = (bitField0_ & ~0x00000002);
          }
          result.controlMessage_ = controlMessage_;
        } else {
          result.controlMessage_ = controlMessageBuilder_.build();
        }
        onBuilt();
        return result;
      }

      @java.lang.Override
      public Builder clone() {
        return super.clone();
      }
      @java.lang.Override
      public Builder setField(
          com.google.protobuf.Descriptors.FieldDescriptor field,
          java.lang.Object value) {
        return super.setField(field, value);
      }
      @java.lang.Override
      public Builder clearField(
          com.google.protobuf.Descriptors.FieldDescriptor field) {
        return super.clearField(field);
      }
      @java.lang.Override
      public Builder clearOneof(
          com.google.protobuf.Descriptors.OneofDescriptor oneof) {
        return super.clearOneof(oneof);
      }
      @java.lang.Override
      public Builder setRepeatedField(
          com.google.protobuf.Descriptors.FieldDescriptor field,
          int index, java.lang.Object value) {
        return super.setRepeatedField(field, index, value);
      }
      @java.lang.Override
      public Builder addRepeatedField(
          com.google.protobuf.Descriptors.FieldDescriptor field,
          java.lang.Object value) {
        return super.addRepeatedField(field, value);
      }
      @java.lang.Override
      public Builder mergeFrom(com.google.protobuf.Message other) {
        if (other instanceof com.acme.glup.ExampleProtoAcme.ClickCas) {
          return mergeFrom((com.acme.glup.ExampleProtoAcme.ClickCas)other);
        } else {
          super.mergeFrom(other);
          return this;
        }
      }

      public Builder mergeFrom(com.acme.glup.ExampleProtoAcme.ClickCas other) {
        if (other == com.acme.glup.ExampleProtoAcme.ClickCas.getDefaultInstance()) return this;
        if (other.hasGlupOrigin()) {
          mergeGlupOrigin(other.getGlupOrigin());
        }
        if (other.hasPartition()) {
          mergePartition(other.getPartition());
        }
        if (!other.getUid().isEmpty()) {
          uid_ = other.uid_;
          onChanged();
        }
        internalGetMutableSetFields().mergeFrom(
            other.internalGetSetFields());
        if (controlMessageBuilder_ == null) {
          if (!other.controlMessage_.isEmpty()) {
            if (controlMessage_.isEmpty()) {
              controlMessage_ = other.controlMessage_;
              bitField0_ = (bitField0_ & ~0x00000002);
            } else {
              ensureControlMessageIsMutable();
              controlMessage_.addAll(other.controlMessage_);
            }
            onChanged();
          }
        } else {
          if (!other.controlMessage_.isEmpty()) {
            if (controlMessageBuilder_.isEmpty()) {
              controlMessageBuilder_.dispose();
              controlMessageBuilder_ = null;
              controlMessage_ = other.controlMessage_;
              bitField0_ = (bitField0_ & ~0x00000002);
              controlMessageBuilder_ = 
                com.google.protobuf.GeneratedMessageV3.alwaysUseFieldBuilders ?
                   getControlMessageFieldBuilder() : null;
            } else {
              controlMessageBuilder_.addAllMessages(other.controlMessage_);
            }
          }
        }
        this.mergeUnknownFields(other.unknownFields);
        onChanged();
        return this;
      }

      @java.lang.Override
      public final boolean isInitialized() {
        return true;
      }

      @java.lang.Override
      public Builder mergeFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws java.io.IOException {
        com.acme.glup.ExampleProtoAcme.ClickCas parsedMessage = null;
        try {
          parsedMessage = PARSER.parsePartialFrom(input, extensionRegistry);
        } catch (com.google.protobuf.InvalidProtocolBufferException e) {
          parsedMessage = (com.acme.glup.ExampleProtoAcme.ClickCas) e.getUnfinishedMessage();
          throw e.unwrapIOException();
        } finally {
          if (parsedMessage != null) {
            mergeFrom(parsedMessage);
          }
        }
        return this;
      }
      private int bitField0_;

      private com.acme.glup.MetadataProto.Origin glupOrigin_;
      private com.google.protobuf.SingleFieldBuilderV3<
          com.acme.glup.MetadataProto.Origin, com.acme.glup.MetadataProto.Origin.Builder, com.acme.glup.MetadataProto.OriginOrBuilder> glupOriginBuilder_;
      /**
       * <code>.Acme.Glup.Origin glup_origin = 1;</code>
       * @return Whether the glupOrigin field is set.
       */
      public boolean hasGlupOrigin() {
        return glupOriginBuilder_ != null || glupOrigin_ != null;
      }
      /**
       * <code>.Acme.Glup.Origin glup_origin = 1;</code>
       * @return The glupOrigin.
       */
      public com.acme.glup.MetadataProto.Origin getGlupOrigin() {
        if (glupOriginBuilder_ == null) {
          return glupOrigin_ == null ? com.acme.glup.MetadataProto.Origin.getDefaultInstance() : glupOrigin_;
        } else {
          return glupOriginBuilder_.getMessage();
        }
      }
      /**
       * <code>.Acme.Glup.Origin glup_origin = 1;</code>
       */
      public Builder setGlupOrigin(com.acme.glup.MetadataProto.Origin value) {
        if (glupOriginBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          glupOrigin_ = value;
          onChanged();
        } else {
          glupOriginBuilder_.setMessage(value);
        }

        return this;
      }
      /**
       * <code>.Acme.Glup.Origin glup_origin = 1;</code>
       */
      public Builder setGlupOrigin(
          com.acme.glup.MetadataProto.Origin.Builder builderForValue) {
        if (glupOriginBuilder_ == null) {
          glupOrigin_ = builderForValue.build();
          onChanged();
        } else {
          glupOriginBuilder_.setMessage(builderForValue.build());
        }

        return this;
      }
      /**
       * <code>.Acme.Glup.Origin glup_origin = 1;</code>
       */
      public Builder mergeGlupOrigin(com.acme.glup.MetadataProto.Origin value) {
        if (glupOriginBuilder_ == null) {
          if (glupOrigin_ != null) {
            glupOrigin_ =
              com.acme.glup.MetadataProto.Origin.newBuilder(glupOrigin_).mergeFrom(value).buildPartial();
          } else {
            glupOrigin_ = value;
          }
          onChanged();
        } else {
          glupOriginBuilder_.mergeFrom(value);
        }

        return this;
      }
      /**
       * <code>.Acme.Glup.Origin glup_origin = 1;</code>
       */
      public Builder clearGlupOrigin() {
        if (glupOriginBuilder_ == null) {
          glupOrigin_ = null;
          onChanged();
        } else {
          glupOrigin_ = null;
          glupOriginBuilder_ = null;
        }

        return this;
      }
      /**
       * <code>.Acme.Glup.Origin glup_origin = 1;</code>
       */
      public com.acme.glup.MetadataProto.Origin.Builder getGlupOriginBuilder() {
        
        onChanged();
        return getGlupOriginFieldBuilder().getBuilder();
      }
      /**
       * <code>.Acme.Glup.Origin glup_origin = 1;</code>
       */
      public com.acme.glup.MetadataProto.OriginOrBuilder getGlupOriginOrBuilder() {
        if (glupOriginBuilder_ != null) {
          return glupOriginBuilder_.getMessageOrBuilder();
        } else {
          return glupOrigin_ == null ?
              com.acme.glup.MetadataProto.Origin.getDefaultInstance() : glupOrigin_;
        }
      }
      /**
       * <code>.Acme.Glup.Origin glup_origin = 1;</code>
       */
      private com.google.protobuf.SingleFieldBuilderV3<
          com.acme.glup.MetadataProto.Origin, com.acme.glup.MetadataProto.Origin.Builder, com.acme.glup.MetadataProto.OriginOrBuilder> 
          getGlupOriginFieldBuilder() {
        if (glupOriginBuilder_ == null) {
          glupOriginBuilder_ = new com.google.protobuf.SingleFieldBuilderV3<
              com.acme.glup.MetadataProto.Origin, com.acme.glup.MetadataProto.Origin.Builder, com.acme.glup.MetadataProto.OriginOrBuilder>(
                  getGlupOrigin(),
                  getParentForChildren(),
                  isClean());
          glupOrigin_ = null;
        }
        return glupOriginBuilder_;
      }

      private com.acme.glup.MetadataProto.Partition partition_;
      private com.google.protobuf.SingleFieldBuilderV3<
          com.acme.glup.MetadataProto.Partition, com.acme.glup.MetadataProto.Partition.Builder, com.acme.glup.MetadataProto.PartitionOrBuilder> partitionBuilder_;
      /**
       * <code>.Acme.Glup.Partition partition = 2;</code>
       * @return Whether the partition field is set.
       */
      public boolean hasPartition() {
        return partitionBuilder_ != null || partition_ != null;
      }
      /**
       * <code>.Acme.Glup.Partition partition = 2;</code>
       * @return The partition.
       */
      public com.acme.glup.MetadataProto.Partition getPartition() {
        if (partitionBuilder_ == null) {
          return partition_ == null ? com.acme.glup.MetadataProto.Partition.getDefaultInstance() : partition_;
        } else {
          return partitionBuilder_.getMessage();
        }
      }
      /**
       * <code>.Acme.Glup.Partition partition = 2;</code>
       */
      public Builder setPartition(com.acme.glup.MetadataProto.Partition value) {
        if (partitionBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          partition_ = value;
          onChanged();
        } else {
          partitionBuilder_.setMessage(value);
        }

        return this;
      }
      /**
       * <code>.Acme.Glup.Partition partition = 2;</code>
       */
      public Builder setPartition(
          com.acme.glup.MetadataProto.Partition.Builder builderForValue) {
        if (partitionBuilder_ == null) {
          partition_ = builderForValue.build();
          onChanged();
        } else {
          partitionBuilder_.setMessage(builderForValue.build());
        }

        return this;
      }
      /**
       * <code>.Acme.Glup.Partition partition = 2;</code>
       */
      public Builder mergePartition(com.acme.glup.MetadataProto.Partition value) {
        if (partitionBuilder_ == null) {
          if (partition_ != null) {
            partition_ =
              com.acme.glup.MetadataProto.Partition.newBuilder(partition_).mergeFrom(value).buildPartial();
          } else {
            partition_ = value;
          }
          onChanged();
        } else {
          partitionBuilder_.mergeFrom(value);
        }

        return this;
      }
      /**
       * <code>.Acme.Glup.Partition partition = 2;</code>
       */
      public Builder clearPartition() {
        if (partitionBuilder_ == null) {
          partition_ = null;
          onChanged();
        } else {
          partition_ = null;
          partitionBuilder_ = null;
        }

        return this;
      }
      /**
       * <code>.Acme.Glup.Partition partition = 2;</code>
       */
      public com.acme.glup.MetadataProto.Partition.Builder getPartitionBuilder() {
        
        onChanged();
        return getPartitionFieldBuilder().getBuilder();
      }
      /**
       * <code>.Acme.Glup.Partition partition = 2;</code>
       */
      public com.acme.glup.MetadataProto.PartitionOrBuilder getPartitionOrBuilder() {
        if (partitionBuilder_ != null) {
          return partitionBuilder_.getMessageOrBuilder();
        } else {
          return partition_ == null ?
              com.acme.glup.MetadataProto.Partition.getDefaultInstance() : partition_;
        }
      }
      /**
       * <code>.Acme.Glup.Partition partition = 2;</code>
       */
      private com.google.protobuf.SingleFieldBuilderV3<
          com.acme.glup.MetadataProto.Partition, com.acme.glup.MetadataProto.Partition.Builder, com.acme.glup.MetadataProto.PartitionOrBuilder> 
          getPartitionFieldBuilder() {
        if (partitionBuilder_ == null) {
          partitionBuilder_ = new com.google.protobuf.SingleFieldBuilderV3<
              com.acme.glup.MetadataProto.Partition, com.acme.glup.MetadataProto.Partition.Builder, com.acme.glup.MetadataProto.PartitionOrBuilder>(
                  getPartition(),
                  getParentForChildren(),
                  isClean());
          partition_ = null;
        }
        return partitionBuilder_;
      }

      private java.lang.Object uid_ = "";
      /**
       * <code>string uid = 5;</code>
       * @return The uid.
       */
      public java.lang.String getUid() {
        java.lang.Object ref = uid_;
        if (!(ref instanceof java.lang.String)) {
          com.google.protobuf.ByteString bs =
              (com.google.protobuf.ByteString) ref;
          java.lang.String s = bs.toStringUtf8();
          uid_ = s;
          return s;
        } else {
          return (java.lang.String) ref;
        }
      }
      /**
       * <code>string uid = 5;</code>
       * @return The bytes for uid.
       */
      public com.google.protobuf.ByteString
          getUidBytes() {
        java.lang.Object ref = uid_;
        if (ref instanceof String) {
          com.google.protobuf.ByteString b = 
              com.google.protobuf.ByteString.copyFromUtf8(
                  (java.lang.String) ref);
          uid_ = b;
          return b;
        } else {
          return (com.google.protobuf.ByteString) ref;
        }
      }
      /**
       * <code>string uid = 5;</code>
       * @param value The uid to set.
       * @return This builder for chaining.
       */
      public Builder setUid(
          java.lang.String value) {
        if (value == null) {
    throw new NullPointerException();
  }
  
        uid_ = value;
        onChanged();
        return this;
      }
      /**
       * <code>string uid = 5;</code>
       * @return This builder for chaining.
       */
      public Builder clearUid() {
        
        uid_ = getDefaultInstance().getUid();
        onChanged();
        return this;
      }
      /**
       * <code>string uid = 5;</code>
       * @param value The bytes for uid to set.
       * @return This builder for chaining.
       */
      public Builder setUidBytes(
          com.google.protobuf.ByteString value) {
        if (value == null) {
    throw new NullPointerException();
  }
  checkByteStringIsUtf8(value);
        
        uid_ = value;
        onChanged();
        return this;
      }

      private com.google.protobuf.MapField<
          java.lang.Integer, java.lang.Boolean> setFields_;
      private com.google.protobuf.MapField<java.lang.Integer, java.lang.Boolean>
      internalGetSetFields() {
        if (setFields_ == null) {
          return com.google.protobuf.MapField.emptyMapField(
              SetFieldsDefaultEntryHolder.defaultEntry);
        }
        return setFields_;
      }
      private com.google.protobuf.MapField<java.lang.Integer, java.lang.Boolean>
      internalGetMutableSetFields() {
        onChanged();;
        if (setFields_ == null) {
          setFields_ = com.google.protobuf.MapField.newMapField(
              SetFieldsDefaultEntryHolder.defaultEntry);
        }
        if (!setFields_.isMutable()) {
          setFields_ = setFields_.copy();
        }
        return setFields_;
      }

      public int getSetFieldsCount() {
        return internalGetSetFields().getMap().size();
      }
      /**
       * <pre>
       * Schema field
       * (...)
       * </pre>
       *
       * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
       */

      public boolean containsSetFields(
          int key) {
        
        return internalGetSetFields().getMap().containsKey(key);
      }
      /**
       * Use {@link #getSetFieldsMap()} instead.
       */
      @java.lang.Deprecated
      public java.util.Map<java.lang.Integer, java.lang.Boolean> getSetFields() {
        return getSetFieldsMap();
      }
      /**
       * <pre>
       * Schema field
       * (...)
       * </pre>
       *
       * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
       */

      public java.util.Map<java.lang.Integer, java.lang.Boolean> getSetFieldsMap() {
        return internalGetSetFields().getMap();
      }
      /**
       * <pre>
       * Schema field
       * (...)
       * </pre>
       *
       * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
       */

      public boolean getSetFieldsOrDefault(
          int key,
          boolean defaultValue) {
        
        java.util.Map<java.lang.Integer, java.lang.Boolean> map =
            internalGetSetFields().getMap();
        return map.containsKey(key) ? map.get(key) : defaultValue;
      }
      /**
       * <pre>
       * Schema field
       * (...)
       * </pre>
       *
       * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
       */

      public boolean getSetFieldsOrThrow(
          int key) {
        
        java.util.Map<java.lang.Integer, java.lang.Boolean> map =
            internalGetSetFields().getMap();
        if (!map.containsKey(key)) {
          throw new java.lang.IllegalArgumentException();
        }
        return map.get(key);
      }

      public Builder clearSetFields() {
        internalGetMutableSetFields().getMutableMap()
            .clear();
        return this;
      }
      /**
       * <pre>
       * Schema field
       * (...)
       * </pre>
       *
       * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
       */

      public Builder removeSetFields(
          int key) {
        
        internalGetMutableSetFields().getMutableMap()
            .remove(key);
        return this;
      }
      /**
       * Use alternate mutation accessors instead.
       */
      @java.lang.Deprecated
      public java.util.Map<java.lang.Integer, java.lang.Boolean>
      getMutableSetFields() {
        return internalGetMutableSetFields().getMutableMap();
      }
      /**
       * <pre>
       * Schema field
       * (...)
       * </pre>
       *
       * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
       */
      public Builder putSetFields(
          int key,
          boolean value) {
        
        
        internalGetMutableSetFields().getMutableMap()
            .put(key, value);
        return this;
      }
      /**
       * <pre>
       * Schema field
       * (...)
       * </pre>
       *
       * <code>map&lt;int32, bool&gt; set_fields = 50010;</code>
       */

      public Builder putAllSetFields(
          java.util.Map<java.lang.Integer, java.lang.Boolean> values) {
        internalGetMutableSetFields().getMutableMap()
            .putAll(values);
        return this;
      }

      private java.util.List<com.acme.glup.MetadataProto.ControlMessage.Watermark> controlMessage_ =
        java.util.Collections.emptyList();
      private void ensureControlMessageIsMutable() {
        if (!((bitField0_ & 0x00000002) != 0)) {
          controlMessage_ = new java.util.ArrayList<com.acme.glup.MetadataProto.ControlMessage.Watermark>(controlMessage_);
          bitField0_ |= 0x00000002;
         }
      }

      private com.google.protobuf.RepeatedFieldBuilderV3<
          com.acme.glup.MetadataProto.ControlMessage.Watermark, com.acme.glup.MetadataProto.ControlMessage.Watermark.Builder, com.acme.glup.MetadataProto.ControlMessage.WatermarkOrBuilder> controlMessageBuilder_;

      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public java.util.List<com.acme.glup.MetadataProto.ControlMessage.Watermark> getControlMessageList() {
        if (controlMessageBuilder_ == null) {
          return java.util.Collections.unmodifiableList(controlMessage_);
        } else {
          return controlMessageBuilder_.getMessageList();
        }
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public int getControlMessageCount() {
        if (controlMessageBuilder_ == null) {
          return controlMessage_.size();
        } else {
          return controlMessageBuilder_.getCount();
        }
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public com.acme.glup.MetadataProto.ControlMessage.Watermark getControlMessage(int index) {
        if (controlMessageBuilder_ == null) {
          return controlMessage_.get(index);
        } else {
          return controlMessageBuilder_.getMessage(index);
        }
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public Builder setControlMessage(
          int index, com.acme.glup.MetadataProto.ControlMessage.Watermark value) {
        if (controlMessageBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureControlMessageIsMutable();
          controlMessage_.set(index, value);
          onChanged();
        } else {
          controlMessageBuilder_.setMessage(index, value);
        }
        return this;
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public Builder setControlMessage(
          int index, com.acme.glup.MetadataProto.ControlMessage.Watermark.Builder builderForValue) {
        if (controlMessageBuilder_ == null) {
          ensureControlMessageIsMutable();
          controlMessage_.set(index, builderForValue.build());
          onChanged();
        } else {
          controlMessageBuilder_.setMessage(index, builderForValue.build());
        }
        return this;
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public Builder addControlMessage(com.acme.glup.MetadataProto.ControlMessage.Watermark value) {
        if (controlMessageBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureControlMessageIsMutable();
          controlMessage_.add(value);
          onChanged();
        } else {
          controlMessageBuilder_.addMessage(value);
        }
        return this;
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public Builder addControlMessage(
          int index, com.acme.glup.MetadataProto.ControlMessage.Watermark value) {
        if (controlMessageBuilder_ == null) {
          if (value == null) {
            throw new NullPointerException();
          }
          ensureControlMessageIsMutable();
          controlMessage_.add(index, value);
          onChanged();
        } else {
          controlMessageBuilder_.addMessage(index, value);
        }
        return this;
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public Builder addControlMessage(
          com.acme.glup.MetadataProto.ControlMessage.Watermark.Builder builderForValue) {
        if (controlMessageBuilder_ == null) {
          ensureControlMessageIsMutable();
          controlMessage_.add(builderForValue.build());
          onChanged();
        } else {
          controlMessageBuilder_.addMessage(builderForValue.build());
        }
        return this;
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public Builder addControlMessage(
          int index, com.acme.glup.MetadataProto.ControlMessage.Watermark.Builder builderForValue) {
        if (controlMessageBuilder_ == null) {
          ensureControlMessageIsMutable();
          controlMessage_.add(index, builderForValue.build());
          onChanged();
        } else {
          controlMessageBuilder_.addMessage(index, builderForValue.build());
        }
        return this;
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public Builder addAllControlMessage(
          java.lang.Iterable<? extends com.acme.glup.MetadataProto.ControlMessage.Watermark> values) {
        if (controlMessageBuilder_ == null) {
          ensureControlMessageIsMutable();
          com.google.protobuf.AbstractMessageLite.Builder.addAll(
              values, controlMessage_);
          onChanged();
        } else {
          controlMessageBuilder_.addAllMessages(values);
        }
        return this;
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public Builder clearControlMessage() {
        if (controlMessageBuilder_ == null) {
          controlMessage_ = java.util.Collections.emptyList();
          bitField0_ = (bitField0_ & ~0x00000002);
          onChanged();
        } else {
          controlMessageBuilder_.clear();
        }
        return this;
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public Builder removeControlMessage(int index) {
        if (controlMessageBuilder_ == null) {
          ensureControlMessageIsMutable();
          controlMessage_.remove(index);
          onChanged();
        } else {
          controlMessageBuilder_.remove(index);
        }
        return this;
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public com.acme.glup.MetadataProto.ControlMessage.Watermark.Builder getControlMessageBuilder(
          int index) {
        return getControlMessageFieldBuilder().getBuilder(index);
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public com.acme.glup.MetadataProto.ControlMessage.WatermarkOrBuilder getControlMessageOrBuilder(
          int index) {
        if (controlMessageBuilder_ == null) {
          return controlMessage_.get(index);  } else {
          return controlMessageBuilder_.getMessageOrBuilder(index);
        }
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public java.util.List<? extends com.acme.glup.MetadataProto.ControlMessage.WatermarkOrBuilder> 
           getControlMessageOrBuilderList() {
        if (controlMessageBuilder_ != null) {
          return controlMessageBuilder_.getMessageOrBuilderList();
        } else {
          return java.util.Collections.unmodifiableList(controlMessage_);
        }
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public com.acme.glup.MetadataProto.ControlMessage.Watermark.Builder addControlMessageBuilder() {
        return getControlMessageFieldBuilder().addBuilder(
            com.acme.glup.MetadataProto.ControlMessage.Watermark.getDefaultInstance());
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public com.acme.glup.MetadataProto.ControlMessage.Watermark.Builder addControlMessageBuilder(
          int index) {
        return getControlMessageFieldBuilder().addBuilder(
            index, com.acme.glup.MetadataProto.ControlMessage.Watermark.getDefaultInstance());
      }
      /**
       * <pre>
       * standard glup field
       * </pre>
       *
       * <code>repeated .Acme.Glup.ControlMessage.Watermark control_message = 2097151 [(.Acme.Glup.json) = { ... }</code>
       */
      public java.util.List<com.acme.glup.MetadataProto.ControlMessage.Watermark.Builder> 
           getControlMessageBuilderList() {
        return getControlMessageFieldBuilder().getBuilderList();
      }
      private com.google.protobuf.RepeatedFieldBuilderV3<
          com.acme.glup.MetadataProto.ControlMessage.Watermark, com.acme.glup.MetadataProto.ControlMessage.Watermark.Builder, com.acme.glup.MetadataProto.ControlMessage.WatermarkOrBuilder> 
          getControlMessageFieldBuilder() {
        if (controlMessageBuilder_ == null) {
          controlMessageBuilder_ = new com.google.protobuf.RepeatedFieldBuilderV3<
              com.acme.glup.MetadataProto.ControlMessage.Watermark, com.acme.glup.MetadataProto.ControlMessage.Watermark.Builder, com.acme.glup.MetadataProto.ControlMessage.WatermarkOrBuilder>(
                  controlMessage_,
                  ((bitField0_ & 0x00000002) != 0),
                  getParentForChildren(),
                  isClean());
          controlMessage_ = null;
        }
        return controlMessageBuilder_;
      }
      @java.lang.Override
      public final Builder setUnknownFields(
          final com.google.protobuf.UnknownFieldSet unknownFields) {
        return super.setUnknownFields(unknownFields);
      }

      @java.lang.Override
      public final Builder mergeUnknownFields(
          final com.google.protobuf.UnknownFieldSet unknownFields) {
        return super.mergeUnknownFields(unknownFields);
      }


      // @@protoc_insertion_point(builder_scope:Acme.Glup.ClickCas)
    }

    // @@protoc_insertion_point(class_scope:Acme.Glup.ClickCas)
    private static final com.acme.glup.ExampleProtoAcme.ClickCas DEFAULT_INSTANCE;
    static {
      DEFAULT_INSTANCE = new com.acme.glup.ExampleProtoAcme.ClickCas();
    }

    public static com.acme.glup.ExampleProtoAcme.ClickCas getDefaultInstance() {
      return DEFAULT_INSTANCE;
    }

    private static final com.google.protobuf.Parser<ClickCas>
        PARSER = new com.google.protobuf.AbstractParser<ClickCas>() {
      @java.lang.Override
      public ClickCas parsePartialFrom(
          com.google.protobuf.CodedInputStream input,
          com.google.protobuf.ExtensionRegistryLite extensionRegistry)
          throws com.google.protobuf.InvalidProtocolBufferException {
        return new ClickCas(input, extensionRegistry);
      }
    };

    public static com.google.protobuf.Parser<ClickCas> parser() {
      return PARSER;
    }

    @java.lang.Override
    public com.google.protobuf.Parser<ClickCas> getParserForType() {
      return PARSER;
    }

    @java.lang.Override
    public com.acme.glup.ExampleProtoAcme.ClickCas getDefaultInstanceForType() {
      return DEFAULT_INSTANCE;
    }

  }

  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_Acme_Glup_ClickCas_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessageV3.FieldAccessorTable
      internal_static_Acme_Glup_ClickCas_fieldAccessorTable;
  private static final com.google.protobuf.Descriptors.Descriptor
    internal_static_Acme_Glup_ClickCas_SetFieldsEntry_descriptor;
  private static final 
    com.google.protobuf.GeneratedMessageV3.FieldAccessorTable
      internal_static_Acme_Glup_ClickCas_SetFieldsEntry_fieldAccessorTable;

  public static com.google.protobuf.Descriptors.FileDescriptor
      getDescriptor() {
    return descriptor;
  }
  private static  com.google.protobuf.Descriptors.FileDescriptor
      descriptor;
  static {
    java.lang.String[] descriptorData = {
      "\n\026exampleProtoAcme.proto\022\tAcme.Glup\032\024met" +
      "adata_proto.proto\032\022common_proto.proto\"\353\002" +
      "\n\010ClickCas\022&\n\013glup_origin\030\001 \001(\0132\021.Acme.G" +
      "lup.Origin\022\'\n\tpartition\030\002 \001(\0132\024.Acme.Glu" +
      "p.Partition\022\013\n\003uid\030\005 \001(\t\0228\n\nset_fields\030\332" +
      "\206\003 \003(\0132\".Acme.Glup.ClickCas.SetFieldsEnt" +
      "ry\022P\n\017control_message\030\377\377\177 \003(\0132#.Acme.Glu" +
      "p.ControlMessage.WatermarkB\020\222\265\030\014\n\n__meta" +
      "data\0320\n\016SetFieldsEntry\022\013\n\003key\030\001 \001(\005\022\r\n\005v" +
      "alue\030\002 \001(\010:\0028\001:\"\210\265\030\001\202\265\030\004:\002\020\001\202\265\030\022\n\020\n\016glup" +
      "_click_casJ\004\010F\020JJ\004\010K\020LR\010obsoleteR\tobsole" +
      "te2B\017\n\rcom.acme.glupb\006proto3"
    };
    descriptor = com.google.protobuf.Descriptors.FileDescriptor
      .internalBuildGeneratedFileFrom(descriptorData,
        new com.google.protobuf.Descriptors.FileDescriptor[] {
          com.acme.glup.MetadataProto.getDescriptor(),
          com.acme.glup.CommonProto.getDescriptor(),
        });
    internal_static_Acme_Glup_ClickCas_descriptor =
      getDescriptor().getMessageTypes().get(0);
    internal_static_Acme_Glup_ClickCas_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessageV3.FieldAccessorTable(
        internal_static_Acme_Glup_ClickCas_descriptor,
        new java.lang.String[] { "GlupOrigin", "Partition", "Uid", "SetFields", "ControlMessage", });
    internal_static_Acme_Glup_ClickCas_SetFieldsEntry_descriptor =
      internal_static_Acme_Glup_ClickCas_descriptor.getNestedTypes().get(0);
    internal_static_Acme_Glup_ClickCas_SetFieldsEntry_fieldAccessorTable = new
      com.google.protobuf.GeneratedMessageV3.FieldAccessorTable(
        internal_static_Acme_Glup_ClickCas_SetFieldsEntry_descriptor,
        new java.lang.String[] { "Key", "Value", });
    com.google.protobuf.ExtensionRegistry registry =
        com.google.protobuf.ExtensionRegistry.newInstance();
    registry.add(com.acme.glup.MetadataProto.containsNullableFields);
    registry.add(com.acme.glup.MetadataProto.glup);
    registry.add(com.acme.glup.MetadataProto.json);
    com.google.protobuf.Descriptors.FileDescriptor
        .internalUpdateFileDescriptor(descriptor, registry);
    com.acme.glup.MetadataProto.getDescriptor();
    com.acme.glup.CommonProto.getDescriptor();
  }

  // @@protoc_insertion_point(outer_class_scope)
}
